<?php

/**
 * Plugin Name:       MF GA4 cookie traking
 * Plugin URI:        https://mario-flores.com
 * Description:       Count cookie usages in page views
 * Version:           1.0.3
 * Requires at least: 5.2
 * Requires PHP:      7.0
 * Author:            Mario Flores
 * Author URI:        https://mario-flores.com
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       mf_gacookies
 * Domain Path:       /languages
 */

 function gcd_create_table() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'cookiecount';

    $charset_collate = $wpdb->get_charset_collate();

    $sql = "CREATE TABLE $table_name (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        date datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
        accepted tinyint(1) DEFAULT 0,
        page varchar(255) DEFAULT '',
        ip_address varchar(45) DEFAULT '',
        PRIMARY KEY (id)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql);
}


register_activation_hook(__FILE__, 'gcd_create_table');

function gcd_check_ga_cookie() {
    if (is_admin()) {
        return;
    }
    
    global $wpdb;
    $table_name = $wpdb->prefix . 'cookiecount';

    // Detecting the GA cookie
    $has_ga_cookie = isset($_COOKIE['_ga']) ? 1 : 0;

    // Get the current page URL
    $current_page_url = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";

    // Get the IP address of the user
    $ip_address = $_SERVER['REMOTE_ADDR'];

    $wpdb->insert(
        $table_name,
        array(
            'date' => current_time('mysql'),
            'accepted' => $has_ga_cookie,
            'page' => $current_page_url,
            'ip_address' => $ip_address
        )
    );
}


add_action('init', 'gcd_check_ga_cookie');

function gcd_drop_table() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'cookiecount';
    $wpdb->query("DROP TABLE IF EXISTS $table_name");
}

register_deactivation_hook(__FILE__, 'gcd_drop_table');
register_uninstall_hook(__FILE__, 'gcd_drop_table');

add_action('admin_menu', 'gcd_register_menu_page');

function gcd_register_menu_page(){
    add_menu_page( 
        'Cookie Count List', 
        'Cookie Count', 
        'manage_options', 
        'cookie-count-list', 
        'gcd_display_records', 
        'dashicons-visibility', 
        6 
    );
}

function gcd_display_records() {
    global $wpdb;
    $table_name = $wpdb->prefix . 'cookiecount';
    $per_page = 1000;

        // Count distinct IPs that accepted cookies
        $ips_accepted = $wpdb->get_var("SELECT COUNT(DISTINCT ip_address) FROM $table_name WHERE accepted = 1");

        // Count distinct IPs that never accepted cookies
        $ips_never_accepted = $wpdb->get_var("SELECT COUNT(DISTINCT ip_address) FROM $table_name WHERE ip_address NOT IN (SELECT DISTINCT ip_address FROM $table_name WHERE accepted = 1)");
    
        echo '<div class="wrap">';
        echo '<h1>Cookie Count Records</h1>';
        
        // Display IP counts
        echo "<p><strong>Distinct IPs that accepted cookies:</strong> {$ips_accepted}</p>";
        echo "<p><strong>Distinct IPs that never accepted cookies:</strong> {$ips_never_accepted}</p>";
    

    // Fetch total number of records
    $total_records = $wpdb->get_var("SELECT COUNT(*) FROM $table_name where page not like '%wp-cron.php%' ");
    $total_pages = ceil($total_records / $per_page);

    // Get the current page from the URL, default to 1 if not set
    $current_page = isset($_GET['paged']) ? max(1, intval($_GET['paged'])) : 1;
    $offset = ($current_page - 1) * $per_page;

    // Fetch the records for the current page
    $records = $wpdb->get_results($wpdb->prepare("SELECT * FROM $table_name where page not like '%wp-cron.php%' ORDER BY date DESC LIMIT %d OFFSET %d", $per_page, $offset));

    echo '<div class="wrap">';
    echo '<h1>Cookie Count Records</h1>';
    
    // Display records in a table
    echo '<table class="wp-list-table widefat fixed striped">';
    echo '<thead>
            <tr>
                <th style="width:140px;">Date</th>
                <th style="width:60px;">Accepted</th>
                <th>Page</th>
                <th style="width:140px;">IP Address</th>
            </tr>
          </thead>
          <tbody>';
    
    foreach ($records as $record) {
        echo "<tr>";
        echo "<td>{$record->date}</td>";
        echo "<td>" . ($record->accepted ? "Yes" : "No") . "</td>";
        echo "<td>{$record->page}</td>";
        echo "<td>{$record->ip_address}</td>";
        echo "</tr>";
    }

    echo '</tbody></table>';

    // Display pagination links
    $base = add_query_arg('paged', '%#%');
    echo '<div class="tablenav bottom">';
    echo '<div class="tablenav-pages">';
    echo paginate_links(array(
        'base' => $base,
        'format' => '',
        'prev_text' => __('&laquo;'),
        'next_text' => __('&raquo;'),
        'total' => $total_pages,
        'current' => $current_page
    ));
    echo '</div>';
    echo '</div>';

    echo '</div>';
}